;*******************************************************************************************
;* program: osnovni program za UNIPIC-1
;* mikroprocesor: PIC16F628
;* info: ovaj program sadrzi osnovne rutine za displej za UNIPIC-1 i definicije 32 karaktera
;* (plus blanko) za prikaz na displeju. Da bi se na nekom displeju prikazao odgovarajuci
;* karakter, potrebno je u njegovu lokaciju smestiti broj od 0 do 33.
;********************************************************************************************

        list p=16f628, f=inhx8m
        include "p16f628.inc"
        include "reg628.asm"
        __CONFIG _CP_OFF & _WDT_OFF & _PWRTE_ON & _XT_OSC & _BODEN_OFF & _LVP_OFF



wsave   equ 20h
ssave   equ 21h 
psave   equ 22h 

stoti   equ 23h         ; stotinke
sekunda equ 24h         ; sekunde
c1      equ 25h         ; cifra hiljada
c2      equ 26h         ; cifra stotina
c3      equ 27h         ; cifra desetina
c4      equ 28h         ; cifra jedinica
                        ; u ove 4 lokacije se smesta broj iz tabele u zavisnosti koji broj ili
                        ; znak se zeli prikazati na displeju. Npr. da bi se na mestu jedinica
                        ; pojavio broj 6, treba u c4 ubaciti broj 6. Da bi se na mestu desetica
                        ; pojavio minus, u c3 treba ubaciti broj 31.
cifra   equ 29h         ; bit 0-3 odredjuje koja cifra se trenutno ispisuje (ispis=0).
                        ; Ovu promenljivu ne treba programski menjati.
tacka   equ 2ah         ; bit 0-3 odredjuje da li se ispisuje odredjena tacka (ispis=1). Ako je
                        ; potreban prikaz tacke, treba odgovarajuci bit setovati. Bit 4 odredjuje
                        ; da li je vreme za blink ili ne (menja se na pola sekunde) - ne treba ga dirati
blink   equ 2bh         ; bit 0-3 odredjuje da li cifra blinka ili ne. Ako je bit=1, cifra blinka. 
                        ; bit 4-7 odredjuje da li tacka blinka ili ne (blink=1). Ako je ispisana
                        ; tacka i ako ima potrebe da i ona blinka pri stelovanju, treba setovati
                        ; odgovarajuci bit

rotor   equ 2ch         ; pomocna za displej
djnz    equ 2dh         ; koristi se za ispis na displej i ne treba je programski menjati

        org 0
        goto inic
        
        org 4

int           ;pocetak interapt rutine

; Snimamo W, STATUS i PCLATH registar

        movwf   wsave
        swapf   status, w
        clrf    status
        movwf   ssave
        movf    pclath, w
        movwf   psave
        

        incf stoti, same        ; uvecaj brojac za stotinke
        movlw .100              ; proveri da li je stigao do 100
        bcf tacka, 4            ; pripremi ne-blinkanje
        subwf stoti, w          ; da li je dosao do 100?
        btfsc status, c
        bsf tacka, 4            ; ako jeste, pripremi blinkanje
        movlw .200
        subwf stoti, w          ; proveri da li je dostigla vrednost 200
        btfss status, c
        goto nije200
        clrf stoti              ; ako jeste, vrati je na nulu
        incf sekunda, same
nije200
        bsf status, c           ; zbog rotiranja
        rlf cifra, same         ; pripremi za ispis sledece cifre
        btfss cifra, 4          ; ako je preslo zadnju cifru
        swapf cifra, same       ; vrati na ispis prve
        btfss cifra, 0          ; pronadji koja cifra
        movf c1, w              ; je naredna za ispis i
        btfss cifra, 1          ; uzmi stanje
        movf c2, w              ; te cifre i
        btfss cifra, 2          ; smesti je u w
        movf c3, w              ; zbog uzimanja
        btfss cifra, 3          ; vrednosti iz tabele za
        movf c4, w              ; definisanje cifre
        clrf pclath
        call tabela
        movwf rotor             ; sacuvaj definiciju cifre za ispis
                
        movf tacka, w           ; provera da li se ispisuje tacka na toj cifri
        iorlw b'11110000'
        iorwf cifra, w
        xorwf cifra, w
        btfsc status, z         ; z=1 ako se ne ispisuje
        goto nodp
        swapf blink, w          ; provera da li ta tacka blinka
        iorlw b'11110000'
        iorwf cifra, w
        xorwf cifra, w
        btfsc status, z         ; z=1 ako ne blinka
        goto upalidp
        btfss tacka, 4
upalidp
        bsf rotor, 0
nodp
        movlw .8
        movwf djnz
        bcf portb, 5
        bcf portb, 4
rot1
        bcf portb, 6            ; izrotiraj 8 bitova u 74595
        rlf rotor, same
        btfsc status, c
        bsf portb, 6
        bsf portb, 4
        bcf portb, 4
        decfsz djnz, same
        goto rot1
        movlw .15
        iorwf portb, same       ; ugasi displej
        bsf portb, 5            ; postavi novo stanje na 74595 za
        bcf portb, 5            ; sledecu cifru na displeju

        movf blink, w           ; provera da li ta cifra blinka
        iorlw b'11110000'
        iorwf cifra, w
        xorwf cifra, w
        btfsc status, z         ; z ako ne blinka
        goto noblink
        btfsc tacka, 4          ; odredjuje brzinu blinkanja
        goto retint
noblink
        movf cifra, w
        andwf portb, same       ; upali sledecu cifru na displeju
retint

        bcf intcon, t0if        ;povratak iz int

; Restore saved registers
        movf    psave, w
        movwf   pclath
        swapf   ssave, w
        movwf   status
        swapf   wsave, same
        swapf   wsave, w

        retfie                  ; Return from interrupt

;sledi tabela definicija za karaktere
tabela  addwf  pcl, same

;              segment     displej   vrednost za c1-c4
;             ;gafbecdD;
        retlw  b'01111110'      ;0            0
        retlw  b'00010100'      ;1            1
        retlw  b'11011010'      ;2            2
        retlw  b'11010110'      ;3            3
        retlw  b'10110100'      ;4            4
        retlw  b'11100110'      ;5            5
        retlw  b'11101110'      ;6            6
        retlw  b'01010100'      ;7            7
        retlw  b'11111110'      ;8            8
        retlw  b'11110100'      ;9            9
        retlw  b'11111110'      ;A           10
        retlw  b'10101110'      ;b           11
        retlw  b'10001010'      ;c           12
        retlw  b'01101010'      ;C           13
        retlw  b'10011110'      ;d           14
        retlw  b'11101010'      ;E           15
        retlw  b'11101000'      ;F           16
        retlw  b'01101110'      ;G           17
        retlw  b'10101100'      ;h           18
        retlw  b'00000010'      ;i           19
        retlw  b'00010110'      ;j           20
        retlw  b'00101010'      ;L           21
        retlw  b'10001100'      ;n           22
        retlw  b'10001110'      ;o           23
        retlw  b'11111000'      ;P           24
        retlw  b'10001000'      ;r           25
        retlw  b'10101010'      ;t           26
        retlw  b'00001110'      ;u           27
        retlw  b'00111110'      ;U           28
        retlw  b'10111100'      ;H           29
        retlw  b'11011000'      ;?           30
        retlw  b'10000000'      ;-           31
        retlw  b'10000010'      ;=           32
        retlw  b'00000000'      ;blanko      33


;*************************************************************************************
;
;       inicijalizacija
;
;*************************************************************************************
inic
        movlw .7                ;Turn comparators off and
        movwf cmcon             ;enable pins for I/O functions
        bcf status, rp1
        bsf status, rp0         ;banka 1
        movlw b'10000011'       ;preskaler /16
        movwf reg_option
        movlw b'00010111'       ;definise porta - svi ulazi sem RA3
        movwf trisa
        movlw b'00000000'
        movwf trisb             ;definise portb - svi izlazi
        bcf status, rp0         ;banka 0
        clrf porta
        movlw .15               ;rb0-rb3 = 1
        movwf portb
        movlw .254              ;priprema prvu cifru za ispis
        movwf cifra
        clrf tacka
        clrf blink
        movlw b'10100000'       ;dozvoljava tajmer interapt
        movwf intcon

; odavde smestas svoju inicijalizaciju
        clrf c1
        clrf c2
        clrf c3
        clrf c4

;*********************************************************************
;*
;*      glavna petlja
;*
;*********************************************************************
pocetak
        goto pocetak
        
        end